@@ -1,4 +1,7 @@
 //! This module includes an overview of GPIO features available.
+//! Most functionality is included as methods to the
+//! [gpio::Pin struct](https://docs.rs/stm32-hal2/latest/stm32_hal2/gpio/struct.Pin.html).
+//!
 //! For project structure and debugging boilerplate, see the `synax_overview` example.
 
 #![no_main]
@@ -7,6 +10,7 @@
 use core::cell::{Cell, RefCell};
 
 use cortex_m::{
+    delay::Delay,
     interrupt::{self, free, Mutex},
     peripheral::NVIC,
 };
@@ -95,6 +99,8 @@ fn main() -> ! {
         defmt::error!("Unable to configure clocks due to a speed error.")
     };
 
+    let mut delay = Delay::new(cp.SYST, clock_cfg.systick());
+
     // Call a function we've made to help organize our pin setup code.
     setup_pins();
 
@@ -113,8 +119,16 @@ fn main() -> ! {
 
     example_type_sigs(&mut example_output, &mut example_input);
 
+    let state = example_input.get_state(); // eg `PinState::High` or `PinState::Low`
+    let state2 = example_input.is_high(); // eg `true` or `false`.
+
     // Set high.
+    example_output.set_state(PinState::High);
+
+    // Simpler syntax to set state, and use with a delay, provided by the Cortex-M systick.
     example_output.set_high();
+    delay.delay_ms(500);
+    example_output.set_low();
 
     // Unmask interrupt lines associated with the input pins we've configured interrupts
     // for in `setup_pins`.
