@@ -209,7 +209,7 @@ impl Default for SpiConfig {
 
 /// Represents a Serial Peripheral Interface (SPI) peripheral.
 pub struct Spi<R> {
-    regs: R,
+    pub regs: R,
     device: SpiDevice,
     cfg: SpiConfig,
 }
@@ -737,7 +737,7 @@ where
 }
 
 #[cfg(feature = "embedded-hal")]
-#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+// #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> FullDuplex<u8> for Spi<R>
 where
     R: Deref<Target = pac::spi1::RegisterBlock>,
@@ -754,14 +754,14 @@ where
 }
 
 #[cfg(feature = "embedded-hal")]
-#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+// #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> embedded_hal::blocking::spi::transfer::Default<u8> for Spi<R> where
     R: Deref<Target = pac::spi1::RegisterBlock>
 {
 }
 
 #[cfg(feature = "embedded-hal")]
-#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+// #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> embedded_hal::blocking::spi::write::Default<u8> for Spi<R> where
     R: Deref<Target = pac::spi1::RegisterBlock>
 {
