@@ -151,10 +151,10 @@ impl Default for UsartConfig {
 
 /// Represents the USART peripheral, for serial communications.
 pub struct Usart<R> {
-    regs: R,
+    pub regs: R,
     device: UsartDevice,
-    pub baud: u32,
-    pub config: UsartConfig,
+    baud: u32,
+    config: UsartConfig,
 }
 
 impl<R> Usart<R>
@@ -683,10 +683,10 @@ pub enum Error {
 }
 
 #[cfg(feature = "embedded-hal")]
-#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
-impl<U> Read<u8> for Usart<U>
+// #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+impl<R> Read<u8> for Usart<R>
 where
-    U: Deref<Target = pac::usart1::RegisterBlock>,
+    R: Deref<Target = pac::usart1::RegisterBlock>,
 {
     type Error = Error;
 
@@ -714,7 +714,7 @@ where
 }
 
 #[cfg(feature = "embedded-hal")]
-#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+// #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> Write<u8> for Usart<R>
 where
     R: Deref<Target = pac::usart1::RegisterBlock>,
@@ -764,7 +764,7 @@ where
 }
 
 #[cfg(feature = "embedded-hal")]
-#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+// #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> blocking::serial::Write<u8> for Usart<R>
 where
     R: Deref<Target = pac::usart1::RegisterBlock>,
